@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -25,7 +25,7 @@
 /*----------------------------------------------------------------------------*/
 
 /*!
-  \page parameters Input of calculation parameters (C functions: cs_user_parameters.c)
+  \page parameters Input of calculation parameters (C functions in cs_user_parameters.c)
 
   \section cs_user_parameters_h_intro Introduction
 
@@ -42,16 +42,60 @@
 
   \section cs_user_parameters_h_cs_user_model  Base model related options
 
-  Definition of user variables or properties should be defined here,
-  if not already done throught the GUI.
+  Definition of user variables or properties as well as choices of physical
+  models should be done here, if not already done through the GUI.
 
-  Most definitions should be done in the \ref cs_user_parameters
-  function, which is the C equivalent of the \ref usipsu Fortran routine.
+  Choose a turbulent model among the available models
 
-  For example, to force the presence of a boundary temperature field, which
-  may be useful for postprocessing:
+  \snippet cs_user_parameters-base.c turbulence_model_choice
 
-  \snippet cs_user_parameters-base.c param_force_b_temperature
+  Coupled solver for Rij components (when iturb=30, 31 or 32): 0 to switch off, 1 to switch on
+
+  \snippet cs_user_parameters-base.c Rij_coupled_solver_choice
+
+  To set a thermal model (0: none, 1: temperature, 2: entyhalpy, 3: total energy)
+
+  \snippet cs_user_parameters-base.c thermal_model_choice
+
+  Volume of Fluid model with mass transfer Merkle model to take into account
+  vaporization / condensation
+
+  \snippet cs_user_parameters-base.c enable_cavit
+
+  To activate ALE (Arbitrary Lagrangian Eulerian) method
+  (CS_ALE_NONE: switch off, CS_ALE_LEGACY: legacy solver, CS_ALE_CDO: CDO solver)
+
+  \snippet cs_user_parameters-base.c ALE_activation
+
+  The user can add a scalar to be solved
+
+  \snippet cs_user_parameters-base.c scalars_addition
+
+  After adding a scalar, the user can add the variance of this scalar
+
+  \snippet cs_user_parameters-base.c scalars_variance_addition
+
+  Add a user property defined on a mesh location (cells, interior faces, boundary faces or vertices).
+
+  \snippet cs_user_parameters-base.c user_property_addition
+
+  \section cs_user_parameters_h_cs_user_parameters General options
+
+  Choose a time step option
+
+  \snippet cs_user_parameters-base.c time_stepping_options
+
+  Choose a reference time step
+
+  \snippet cs_user_parameters-base.c ref_time_step
+
+  To set a duration
+
+  \snippet cs_user_parameters-base.c duration
+
+  For example, to change the log (run_solver.log) verbosity of all the variables:
+
+  \snippet cs_user_parameters-base.c param_log_verbosity
 
   For example, to change limiters for the convective scheme for a given scalar:
 
@@ -63,15 +107,98 @@
 
   If one wants to declare a scalar as buoyant (i.e. the density depends on this scalar
   through a given equation of state) and add it in the velocity-pressure optional
-  sub-iterations (PISO), one can set the dedicated keyword:
+  inner-iterations, one can set the dedicated keyword:
 
   \snippet cs_user_parameters-base.c param_var_is_buoyant
 
+  If one wants to activate drift terms on a transported scalar:
 
-  To set options for the solving of the Stokes problem (velocity pressure coupling
-  see \ref cs_stokes_model_t structure):
+  \snippet cs_user_parameters-base.c param_var_drift
+
+  To set model and parameter options for the velocity-pressure coupling and
+  resolution, see the \ref cs_velocity_pressure_model_t and
+  \ref cs_velocity_pressure_param_t structures.
+
+  For example, to set the Rhie and Chow multiplication factor:
+
+  \snippet cs_user_parameters-base.c param_vp_arak
+
+  To set the optional number of velocity-pressure inner iterations:
+
+  \snippet cs_user_parameters-base.c param_vp_netrup
+
+  \subsection cs_user_parameters_h_cs_user_parameters1 Special fields and activate some automatic post-processings
+
+  For example, to force the presence of a boundary temperature field, which
+  may be useful for postprocessing:
+
+  \snippet cs_user_parameters-base.c param_force_b_temperature
+
+  To add boundary values for all scalars, the
+  following code can be added:
+
+  \snippet cs_user_parameters-base.c param_var_boundary_vals_1
+
+  To add handling (storage) of previous values for a field, the following
+  following code can be added:
+
+  \snippet cs_user_parameters-base.c user_field_n_time_vals
+
+  Enforce existence of 'tplus' and 'tstar' fields, so that
+  a Nusselt number may be computed using the
+  \ref post_boundary_nusselt subroutine.
+  When postprocessing this quantity is activated, those fields
+  are present, but if we need to compute them in the
+  \ref cs_user_extra_operations user subroutine without postprocessing them,
+  forcing the definition of these fields to save the values computed
+  for the boundary layer is necessary.
+
+  \snippet cs_user_parameters-base.c param_force_yplus
+
+  You can activate the post-processing of the Q-criterion on the whole domain
+  mesh with:
+
+  \snippet cs_user_parameters-base.c param_var_q_criterion
+
+  Save contribution of slope test for variables in special fields.
+  These fields are automatically created, with postprocessing output enabled,
+  if the matching variable is convected, does not use a pure upwind scheme,
+  and has a slope test (the slope_test_upwind_id key value for a given
+  variable's field is automatically set to the matching postprocessing field's
+  id, or -1 if not applicable).
+
+  \snippet cs_user_parameters-base.c param_post_slop_test
+
+  You can activate the post-processing of clipping on turbulent quantities
+  on the whole domain mesh with:
+
+  \snippet cs_user_parameters-base.c param_var_rij_clipping
+
+
+  \section cs_user_parameters_h_finalize_setup Input-output related examples (usipes)
+
+  \subsection cs_user_parameters_h_example_base Basic options
+
+  Frequency of log output.
+
+  \snippet cs_user_parameters-base.c setup_log
+
+  Change a property's label (here for density, first checking if it
+  is variable). A field's name cannot be changed, but its label,
+  used for logging and postprocessing output, may be redefined.
+
+  \snippet cs_user_parameters-base.c setup_label
+
+  \subsection cs_user_parameters_h_example_post Postprocessing output
+
+  Activate or deactivate probes output.
+
+  \snippet cs_user_parameters-base.c setup_post
+
+  Probes for Radiative Transfer (Luminance and radiative density flux vector).
+
+  \snippet cs_user_parameters-base.c setup_post_lum
 
-  \snippet cs_user_parameters-base.c param_stokes_model
 
   \section cs_user_parameters_h_cs_user_model_cdo Base model for CDO/HHO schemes
 
@@ -154,7 +281,7 @@
   the darcean flux as the advection field. This call implies the
   creation of a new equation related to a new variable field name
   given as arguments. Advanced tracer equations can be defined using
-  \ref cs_gwf_add_tracer_user.
+  \ref cs_gwf_add_user_tracer.
 
   \snippet cs_user_parameters-cdo-gwf.c param_cdo_gwf_add_tracer
 
@@ -222,7 +349,7 @@
   \subsection cs_user_parameters_h_sles_ex_1 Example: distance to wall
 
   By default, the wall distance (active only with turbulence models which
-  require it) is computed with a preconditionned conjugate gradient.
+  require it) is computed with a preconditioned conjugate gradient.
   The following example shows how to use a multigrid solver for this
   quantity (useful especially if computed repeatedly, such as for ALE).
 
@@ -369,6 +496,29 @@
 
   \snippet cs_user_parameters-linear_solvers.c sles_petsc_hook_view
 
+  \subsection cs_user_parameters_h_sles_amgx Using AmgX
+
+  The AmgX library provides advanced solvers targeting NVIDIA GPUs.
+
+  As described in its documentation, AmgX solvers can be configured
+  either using a configuration string (containing key/value pairs),
+  or \c .json formatted files.
+
+  The following example shows how to setup a solver to use the AmgX
+  library, if the code was built with AmgX support. In this example,
+  a configuration file (which must be present in the case's \c DATA
+  directory) is used.
+
+  \snippet cs_user_parameters-linear_solvers.c sles_amgx
+
+  To set options using a string, the \ref cs_sles_amgx_set_config
+  function may be used intead of \ref cs_sles_amgx_set_config_file.
+  If neither is called, a defaut configuration is used.
+
+  The \ref cs_sles_amgx_set_pin_memory and \ref cs_sles_amgx_set_use_device
+  functions may also be called to modify default behavior relative to
+  using pinned memory and running on the device or host.
+
   \section cs_user_parameters_h_cs_user_moments  Time moment related options
 
   Code_Saturne allows the calculation of temporal means or variances,
@@ -404,6 +554,12 @@
 
   \snippet cs_user_parameters-time_moments.c tmom_u
 
+  In the next example, we define the variance of the vector velocity.
+  All components are used again (component -1 means all components),
+  so the moment is a tensor.
+
+  \snippet cs_user_parameters-time_moments.c tmom_variance_u
+
   \subsection cs_user_parameters_h_example_2 Example 2
 
   In the next example, we multiply the expression by the density.
@@ -480,6 +636,9 @@
 
   \snippet cs_user_parameters-time_moments.c tmom_velocity_rotation
 
+  To activate means for all variables:
+  \snippet cs_user_parameters-time_moments.c tmom_all_variables
+
   \section cs_user_parameters_h_cs_user_fans  Fan modelling options
 
   Code_Saturne allows modelling of some circular fans as volume
@@ -495,21 +654,4 @@
 
   \snippet cs_user_parameters-fans.c fan_user_1
 
-  \section cs_user_parameters_h_postprocess Activate some automatic post-processings
-
-  To add boundary values for all scalars, the
-  following code can be added:
-
-  \snippet cs_user_parameters-base.c param_var_boundary_vals_1
-
-  You can activate the post-processing of the Q-criterion on the whole domain
-  mesh with:
-
-  \snippet cs_user_parameters-base.c param_var_q_criterion
-
-  You can activate the post-processing of clipping on turbulent quantities
-  on the whole domain mesh with:
-
-  \snippet cs_user_parameters-base.c param_var_rij_clipping
-
 */
