@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -948,7 +948,7 @@ _echo_header(const char     *sec_name,
  * FVM datatypes must have been converted to the corresponding
  * Code_Saturne compatible datatype before calling this function:
  *   CS_CHAR               -> char
- *   CS_INT32 / CS_INT64   -> cs_lnum_t / cs_int_t
+ *   CS_INT32 / CS_INT64   -> cs_lnum_t
  *   CS_UINT32 / CS_UINT64 -> cs_gnum_t
  *   CS_REAL / CS_FLOAT    -> double / cs_real_t
  *
@@ -1023,8 +1023,9 @@ _echo_data(size_t          echo,
         const cs_lnum_t *_elts = elts;
 
         for (i = echo_start ; i < echo_end ; i++)
-          bft_printf("    %10llu : %12d\n",
-                     (unsigned long long)(i + num_shift), *(_elts + i));
+          bft_printf("    %10llu : %12ld\n",
+                     (unsigned long long)(i + num_shift),
+                     (long)*(_elts + i));
       }
       break;
 
@@ -1092,7 +1093,7 @@ _echo_data(size_t          echo,
  * dest_type must have been set to the corresponding
  * Code_Saturne compatible datatype before calling this function and
  * conversion will be done accordingly:
- *   CS_INT32 / CS_INT64   -> cs_lnum_t / cs_int_t
+ *   CS_INT32 / CS_INT64   -> cs_lnum_t
  *   CS_UINT32 / CS_UINT64 -> cs_gnum_t
  *   CS_REAL / CS_FLOAT    -> double / cs_real_t
  *
@@ -1112,7 +1113,6 @@ _cs_io_convert_read(void           *buffer,
                     cs_datatype_t   dest_type)
 {
   cs_file_off_t ii;
-  size_t buffer_type_size = cs_datatype_size[buffer_type];
 
   assert(dest_type != buffer_type);
 
@@ -1123,130 +1123,110 @@ _cs_io_convert_read(void           *buffer,
   switch(dest_type) {
 
   case CS_INT32:
+    {
+      int32_t *_dest = dest;
+
+      if (buffer_type == CS_INT32) {
+        int32_t * _buffer = buffer;
+        for (ii = 0; ii < n_elts; ii++)
+          _dest[ii] = _buffer[ii];
+      }
+      else if (buffer_type == CS_INT64) {
+        int64_t * _buffer = buffer;
+        for (ii = 0; ii < n_elts; ii++)
+          _dest[ii] = _buffer[ii];
+      }
+      if (buffer_type == CS_UINT32) {
+        uint32_t * _buffer = buffer;
+        for (ii = 0; ii < n_elts; ii++)
+          _dest[ii] = _buffer[ii];
+      }
+      else if (buffer_type == CS_UINT64) {
+        uint64_t * _buffer = buffer;
+        for (ii = 0; ii < n_elts; ii++)
+          _dest[ii] = _buffer[ii];
+      }
+    }
+    break;
+
   case CS_INT64:
     {
-      cs_lnum_t *_dest = dest;
+      int64_t *_dest = dest;
 
-      if (   buffer_type == CS_INT32
-          || buffer_type == CS_INT64) {
-
-        if (sizeof(long) == buffer_type_size) {
-          long * _buffer = buffer;
-          for (ii = 0; ii < n_elts; ii++)
-            _dest[ii] = _buffer[ii];
-        }
-        else if (sizeof(long long) == buffer_type_size) {
-          long long * _buffer = buffer;
-          for (ii = 0; ii < n_elts; ii++)
+      if (buffer_type == CS_INT32) {
+        int32_t * _buffer = buffer;
+        for (ii = 0; ii < n_elts; ii++)
           _dest[ii] = _buffer[ii];
-        }
-        else if (sizeof(int) == buffer_type_size) {
-          int * _buffer = buffer;
-          for (ii = 0; ii < n_elts; ii++)
-          _dest[ii] = _buffer[ii];
-        }
-        else if (sizeof(short) == buffer_type_size) {
-          short * _buffer = buffer;
-          for (ii = 0; ii < n_elts; ii++)
-          _dest[ii] = _buffer[ii];
-        }
-
       }
-
-      else if (   buffer_type == CS_UINT32
-               || buffer_type == CS_UINT64) {
-
-        if (sizeof(unsigned long) == buffer_type_size) {
-          unsigned long * _buffer = buffer;
-          for (ii = 0; ii < n_elts; ii++)
-            _dest[ii] = _buffer[ii];
-        }
-        else if (sizeof(unsigned long long) == buffer_type_size) {
-          unsigned long long * _buffer = buffer;
-          for (ii = 0; ii < n_elts; ii++)
+      else if (buffer_type == CS_INT64) {
+        int64_t * _buffer = buffer;
+        for (ii = 0; ii < n_elts; ii++)
+          _dest[ii] = _buffer[ii];
+      }
+      if (buffer_type == CS_UINT32) {
+        uint32_t * _buffer = buffer;
+        for (ii = 0; ii < n_elts; ii++)
+          _dest[ii] = _buffer[ii];
+      }
+      else if (buffer_type == CS_UINT64) {
+        uint64_t * _buffer = buffer;
+        for (ii = 0; ii < n_elts; ii++)
           _dest[ii] = _buffer[ii];
-        }
-        else if (sizeof(unsigned int) == buffer_type_size) {
-          unsigned int * _buffer = buffer;
-          for (ii = 0; ii < n_elts; ii++)
-          _dest[ii] = _buffer[ii];
-        }
-        else if (sizeof(unsigned short) == buffer_type_size) {
-          unsigned short * _buffer = buffer;
-          for (ii = 0; ii < n_elts; ii++)
-          _dest[ii] = _buffer[ii];
-        }
-
       }
-
-      assert(   sizeof(long) == buffer_type_size
-             || sizeof(long long) == buffer_type_size
-             || sizeof(int) == buffer_type_size
-             || sizeof(short) == buffer_type_size);
     }
     break;
 
   case CS_UINT32:
+    {
+      uint32_t *_dest = dest;
+
+      if (buffer_type == CS_INT32) {
+        int32_t * _buffer = buffer;
+        for (ii = 0; ii < n_elts; ii++)
+          _dest[ii] = _buffer[ii];
+      }
+      else if (buffer_type == CS_INT64) {
+        int64_t * _buffer = buffer;
+        for (ii = 0; ii < n_elts; ii++)
+          _dest[ii] = _buffer[ii];
+      }
+      if (buffer_type == CS_UINT32) {
+        uint32_t * _buffer = buffer;
+        for (ii = 0; ii < n_elts; ii++)
+          _dest[ii] = _buffer[ii];
+      }
+      else if (buffer_type == CS_UINT64) {
+        uint64_t * _buffer = buffer;
+        for (ii = 0; ii < n_elts; ii++)
+          _dest[ii] = _buffer[ii];
+      }
+    }
+    break;
+
   case CS_UINT64:
     {
-      cs_gnum_t *_dest = dest;
+      uint64_t *_dest = dest;
 
-      if (   buffer_type == CS_INT32
-          || buffer_type == CS_INT64) {
-
-        if (sizeof(long) == buffer_type_size) {
-          long * _buffer = buffer;
-          for (ii = 0; ii < n_elts; ii++)
-            _dest[ii] = _buffer[ii];
-        }
-        else if (sizeof(long long) == buffer_type_size) {
-          long long * _buffer = buffer;
-          for (ii = 0; ii < n_elts; ii++)
+      if (buffer_type == CS_INT32) {
+        int32_t * _buffer = buffer;
+        for (ii = 0; ii < n_elts; ii++)
           _dest[ii] = _buffer[ii];
-        }
-        else if (sizeof(int) == buffer_type_size) {
-          int * _buffer = buffer;
-          for (ii = 0; ii < n_elts; ii++)
-          _dest[ii] = _buffer[ii];
-        }
-        else if (sizeof(short) == buffer_type_size) {
-          short * _buffer = buffer;
-          for (ii = 0; ii < n_elts; ii++)
-          _dest[ii] = _buffer[ii];
-        }
-
       }
-
-      else if (   buffer_type == CS_UINT32
-               || buffer_type == CS_UINT64) {
-
-        if (sizeof(unsigned long) == buffer_type_size) {
-          unsigned long * _buffer = buffer;
-          for (ii = 0; ii < n_elts; ii++)
-            _dest[ii] = _buffer[ii];
-        }
-        else if (sizeof(unsigned long long) == buffer_type_size) {
-          unsigned long long * _buffer = buffer;
-          for (ii = 0; ii < n_elts; ii++)
+      else if (buffer_type == CS_INT64) {
+        int64_t * _buffer = buffer;
+        for (ii = 0; ii < n_elts; ii++)
+          _dest[ii] = _buffer[ii];
+      }
+      if (buffer_type == CS_UINT32) {
+        uint32_t * _buffer = buffer;
+        for (ii = 0; ii < n_elts; ii++)
+          _dest[ii] = _buffer[ii];
+      }
+      else if (buffer_type == CS_UINT64) {
+        uint64_t * _buffer = buffer;
+        for (ii = 0; ii < n_elts; ii++)
           _dest[ii] = _buffer[ii];
-        }
-        else if (sizeof(unsigned int) == buffer_type_size) {
-          unsigned int * _buffer = buffer;
-          for (ii = 0; ii < n_elts; ii++)
-          _dest[ii] = _buffer[ii];
-        }
-        else if (sizeof(unsigned short) == buffer_type_size) {
-          unsigned short * _buffer = buffer;
-          for (ii = 0; ii < n_elts; ii++)
-          _dest[ii] = _buffer[ii];
-        }
-
       }
-
-      assert(   sizeof(long) == buffer_type_size
-             || sizeof(long long) == buffer_type_size
-             || sizeof(int) == buffer_type_size
-             || sizeof(short) == buffer_type_size);
     }
     break;
 
@@ -1355,7 +1335,7 @@ _cs_io_read_body(const cs_io_sec_header_t  *header,
   _elts = elts;
 
   if (_elts == NULL && n_vals != 0) {
-    if (header->elt_type == CS_CHAR)
+    if (header->elt_type == CS_CHAR && header->location_id == 0)
       BFT_MALLOC(_elts, n_vals + 1, char);
     else
       BFT_MALLOC(_elts, n_vals*type_size, char);
@@ -1447,7 +1427,7 @@ _cs_io_read_body(const cs_io_sec_header_t  *header,
 
   /* Add null character at end of string to ensure C-type string */
 
-  if (n_vals != 0 && header->elt_type == CS_CHAR)
+  if (n_vals != 0 && header->elt_type == CS_CHAR && header->location_id == 0)
     ((char *)_elts)[header->n_vals] = '\0';
 
   if (log != NULL) {
@@ -1639,10 +1619,7 @@ _write_header(const char     *sec_name,
   header_vals[2] = location_id;
   header_vals[3] = index_id;
 
-  if (n_vals > 0)
-    header_vals[4] = n_location_vals;
-  else
-    header_vals[4] = 0;
+  header_vals[4] = n_location_vals;
 
   name_size = strlen(sec_name);
   name_pad_size = 8-(name_size%8); /* At least 1 NULL
@@ -1681,49 +1658,47 @@ _write_header(const char     *sec_name,
 
   outp->type_name = (char *)outp->buffer + 48;
 
-  if (n_vals > 0) {
-    switch(elt_type) {
-    case CS_FLOAT:
-      outp->type_name[0] = 'r';
-      if (sizeof(float) == 4)
-        outp->type_name[1] = '4';
-      else
-        outp->type_name[1] = '8';
-      assert(sizeof(float) == 4 || sizeof(float) == 8);
-      break;
-    case CS_DOUBLE:
-      outp->type_name[0] = 'r';
-      if (sizeof(double) == 8)
-        outp->type_name[1] = '8';
-      else {
-        outp->type_name[1] = '1';
-        outp->type_name[2] = '6';
-      }
-      assert(sizeof(double) == 8 || sizeof(float) == 16);
-      break;
-    case CS_INT32:
-      outp->type_name[0] = 'i';
+  switch(elt_type) {
+  case CS_FLOAT:
+    outp->type_name[0] = 'r';
+    if (sizeof(float) == 4)
       outp->type_name[1] = '4';
-      break;
-    case CS_INT64:
-      outp->type_name[0] = 'i';
+    else
       outp->type_name[1] = '8';
-      break;
-    case CS_UINT32:
-      outp->type_name[0] = 'u';
-      outp->type_name[1] = '4';
-      break;
-    case CS_UINT64:
-      outp->type_name[0] = 'u';
+    assert(sizeof(float) == 4 || sizeof(float) == 8);
+    break;
+  case CS_DOUBLE:
+    outp->type_name[0] = 'r';
+    if (sizeof(double) == 8)
       outp->type_name[1] = '8';
-      break;
-    case CS_CHAR:
-      outp->type_name[0] = 'c';
-      outp->type_name[1] = ' ';
-      break;
-    default:
-      break;
+    else {
+      outp->type_name[1] = '1';
+      outp->type_name[2] = '6';
     }
+    assert(sizeof(double) == 8 || sizeof(float) == 16);
+    break;
+  case CS_INT32:
+    outp->type_name[0] = 'i';
+    outp->type_name[1] = '4';
+    break;
+  case CS_INT64:
+    outp->type_name[0] = 'i';
+    outp->type_name[1] = '8';
+    break;
+  case CS_UINT32:
+    outp->type_name[0] = 'u';
+    outp->type_name[1] = '4';
+    break;
+  case CS_UINT64:
+    outp->type_name[0] = 'u';
+    outp->type_name[1] = '8';
+    break;
+  case CS_CHAR:
+    outp->type_name[0] = 'c';
+    outp->type_name[1] = ' ';
+    break;
+  default:
+    break;
   }
 
   if (embed == true)
@@ -2409,6 +2384,43 @@ cs_io_set_indexed_position(cs_io_t             *inp,
   return retval;
 }
 
+/*----------------------------------------------------------------------------
+ * Set a section's final data type to int.
+ *
+ * It the datatype is not compatible, throw an error.
+ *
+ * parameters:
+ *   header <-- header structure
+ *   cs_io  --> kernel IO structure
+ *----------------------------------------------------------------------------*/
+
+void
+cs_io_set_int(cs_io_sec_header_t  *header,
+              const cs_io_t       *cs_io)
+{
+  assert(header != NULL);
+
+  if (   header->type_read != CS_INT32
+      && header->type_read != CS_INT64
+      && header->type_read != CS_UINT32
+      && header->type_read != CS_UINT64)
+    bft_error(__FILE__, __LINE__, 0,
+              _("Error reading file: \"%s\".\n"
+                "Type expected for section: "
+                "\"%s\" is a signed integer\n"
+                "and is not convertible from type read: \"%s\"."),
+              cs_file_get_name(cs_io->f),
+              header->sec_name,
+              cs_io->type_name);
+
+  assert(sizeof(int) == 4 || sizeof(int) == 8);
+
+  if (sizeof(int) == 4)
+    header->elt_type = CS_INT32;
+  else
+    header->elt_type = CS_INT64;
+}
+
 /*----------------------------------------------------------------------------
  * Set a section's final data type to cs_lnum_t.
  *
