@@ -8,7 +8,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -38,6 +38,7 @@
  *----------------------------------------------------------------------------*/
 
 #include "cs_base.h"
+#include "cs_math.h"
 #include "cs_turbulence_model.h"
 
 /*----------------------------------------------------------------------------*/
@@ -57,21 +58,26 @@ BEGIN_C_DECLS
 
 typedef enum {
 
-  CS_WALL_F_DISABLED,
-  CS_WALL_F_1SCALE_POWER,
-  CS_WALL_F_1SCALE_LOG,
-  CS_WALL_F_2SCALES_LOG,
-  CS_WALL_F_SCALABLE_2SCALES_LOG,
-  CS_WALL_F_2SCALES_VDRIEST,
-  CS_WALL_F_2SCALES_SMOOTH_ROUGH,
-  CS_WALL_F_2SCALES_CONTINUOUS,
+  CS_WALL_F_UNSET = -999,
+  CS_WALL_F_DISABLED = 0,
+  CS_WALL_F_1SCALE_POWER = 1,
+  CS_WALL_F_1SCALE_LOG = 2,
+  CS_WALL_F_2SCALES_LOG = 3,
+  CS_WALL_F_SCALABLE_2SCALES_LOG = 4,
+  CS_WALL_F_2SCALES_VDRIEST = 5,
+  CS_WALL_F_2SCALES_SMOOTH_ROUGH = 6,
+  CS_WALL_F_2SCALES_CONTINUOUS = 7,
 
 } cs_wall_f_type_t;
 
 typedef enum {
 
-  CS_WALL_F_S_ARPACI_LARSEN,
-  CS_WALL_F_S_VDRIEST,
+  CS_WALL_F_S_UNSET = -999,
+  CS_WALL_F_S_ARPACI_LARSEN = 0,
+  CS_WALL_F_S_VDRIEST = 1,
+  CS_WALL_F_S_LOUIS = 2,
+  CS_WALL_F_S_MONIN_OBUKHOV = 3,
+  CS_WALL_F_S_SMOOTH_ROUGH = 4,//TODO merge with MO ?
 
 } cs_wall_f_s_type_t;
 
@@ -262,8 +268,8 @@ cs_wall_functions_1scale_log(cs_lnum_t    ifac,
     if (iter >= niter_max) {
       bft_printf(_("WARNING: non-convergence in the computation\n"
                    "******** of the friction velocity\n\n"
-                   "face number: %d \n"
-                   "friction vel: %f \n" ), ifac, *ustar);
+                   "face id: %ld \n"
+                   "friction vel: %f \n" ), (long)ifac, *ustar);
     }
 
     *uk = *ustar;
@@ -280,26 +286,28 @@ cs_wall_functions_1scale_log(cs_lnum_t    ifac,
 
 /*----------------------------------------------------------------------------
  * Compute du+/dy+ for a given yk+.
+ *
  * parameters:
  *   yplus     <--  dimensionless distance
+ *
  * returns:
  *   the resulting dimensionless velocity.
  *----------------------------------------------------------------------------*/
 
 inline static cs_real_t
-_uplus(cs_real_t yp  ,
-       cs_real_t ka  ,
-       cs_real_t B   ,
-       cs_real_t cuv ,
-       cs_real_t y0  ,
-       cs_real_t n   )
+_uplus(cs_real_t yp,
+       cs_real_t ka,
+       cs_real_t B,
+       cs_real_t cuv,
+       cs_real_t y0,
+       cs_real_t n)
 {
-  cs_real_t uplus, f_blend ;
+  cs_real_t uplus, f_blend;
 
-  f_blend = exp(-0.25*cuv*pow(yp,3)) ;
+  f_blend = exp(-0.25*cuv*pow(yp,3));
   uplus   = f_blend*yp + (log(yp)/ka +B)*(1.-exp(-pow(yp/y0,n)))*(1-f_blend);
 
-  return uplus ;
+  return uplus;
 }
 
 /*----------------------------------------------------------------------------
@@ -311,22 +319,24 @@ _uplus(cs_real_t yp  ,
  *----------------------------------------------------------------------------*/
 
 inline static cs_real_t
-_dupdyp(cs_real_t yp  ,
-        cs_real_t ka  ,
-        cs_real_t B   ,
-        cs_real_t cuv ,
-        cs_real_t y0  ,
-        cs_real_t n   )
+_dupdyp(cs_real_t yp,
+        cs_real_t ka,
+        cs_real_t B,
+        cs_real_t cuv,
+        cs_real_t y0,
+        cs_real_t n)
 {
-  cs_real_t dupdyp ;
+  cs_real_t dupdyp;
 
   dupdyp = exp(-0.25*cuv*pow(yp,3))
    - 0.75*cuv*pow(yp,3.)*exp(-0.25*cuv*pow(yp,3.))
-   + n*(1.-exp(-0.25*cuv*pow(yp,3.)))*(pow(yp,n-1.)/pow(y0,n))*exp(-pow(yp/y0,n))*((1./ka)*log(yp)+B)
-   + 0.75*cuv*pow(yp,2.)*exp(-0.25*cuv*pow(yp,3.))*(1.-exp(-pow(yp/y0,n)))*((1./ka)*log(yp)+B)
-   + (1./ka/yp)*(1.-exp(-pow(yp/y0,n)))*(1-exp(-0.25*cuv*pow(yp,3.))) ;
+   + n*(1.-exp(-0.25*cuv*pow(yp,3.)))*(pow(yp,n-1.)/pow(y0,n))
+      *exp(-pow(yp/y0,n))*((1./ka)*log(yp)+B)
+   + 0.75*cuv*pow(yp,2.)*exp(-0.25*cuv*pow(yp,3.))
+         *(1.-exp(-pow(yp/y0,n)))*((1./ka)*log(yp)+B)
+   + (1./ka/yp)*(1.-exp(-pow(yp/y0,n)))*(1-exp(-0.25*cuv*pow(yp,3.)));
 
-  return dupdyp ;
+  return dupdyp;
 }
 
 /*----------------------------------------------------------------------------*/
@@ -372,8 +382,8 @@ cs_wall_functions_2scales_continuous(cs_real_t   rnnb,
 {
   const double ypluli = cs_glob_wall_functions->ypluli;
   double Re, g, t_visc_durb;
-  cs_real_t cstcuv, csty0, cstN ;
-  cs_real_t dup1, dup2, uplus ;
+  cs_real_t cstcuv, csty0, cstN;
+  cs_real_t dup1, dup2, uplus;
 
   /* Local constants */
   cstcuv = 1.0674e-3;
@@ -386,30 +396,33 @@ cs_wall_functions_2scales_continuous(cs_real_t   rnnb,
 
   /* Comutation of uk*/
   *uk = sqrt( (1.-g) * cs_turb_cmu025 * cs_turb_cmu025 * kinetic_en
-            + g * l_visc * vel / y);
+             + g * l_visc * vel/y);
 
   /* Local value of y+, estimated U+ */
   *yplus = *uk * y / l_visc;
   uplus  = _uplus( *yplus, cs_turb_xkappa, cs_turb_cstlog, cstcuv, csty0, cstN);
   /* Deduced velocity sclale uet*/
-  *ustar = vel / uplus ;
+  *ustar = vel / uplus;
 
-  if( *yplus < 1.e-1 ) {
+  if (*yplus < 1.e-1) {
 
-    *ypup   = 1.0 ;
-    *cofimp = 0.0 ;
+    *ypup   = 1.0;
+    *cofimp = 0.0;
 
     *iuntur = 0;
     *nsubla += 1;
 
-  } else {
+  }
+  else {
 
     /* Dimensionless velocity gradient in y+ */
-    dup1 = _dupdyp(     *yplus, cs_turb_xkappa, cs_turb_cstlog, cstcuv, csty0, cstN);
+    dup1 = _dupdyp(*yplus, cs_turb_xkappa, cs_turb_cstlog,
+                   cstcuv, csty0, cstN);
     /* Dimensionless velocity gradient in 2 x y+ */
-    dup2 = _dupdyp(2.0 * *yplus , cs_turb_xkappa, cs_turb_cstlog, cstcuv, csty0, cstN);
+    dup2 = _dupdyp(2.0 * *yplus, cs_turb_xkappa,
+                   cs_turb_cstlog, cstcuv, csty0, cstN);
 
-    *ypup = *yplus / uplus ;
+    *ypup = *yplus / uplus;
 
     /* ------------------------------------------------------------
      * Cofimp = U,F/U,I is built so that the theoretical expression
@@ -422,13 +435,16 @@ cs_wall_functions_2scales_continuous(cs_real_t   rnnb,
     /* Turbulent viscocity is modified for RSM so that its expression
      * remain valid down to the wall, according to Durbin :
      * nu_t = 0.22 * v'2 * k / eps */
-    if (cs_glob_turb_model->itytur == 3) {
-      t_visc_durb = t_visc / (kinetic_en * cs_turb_cmu ) * rnnb * 0.22 ;
-    } else {
-      t_visc_durb = t_visc ;
-    }
+    const cs_turb_model_t  *turb_model = cs_get_glob_turb_model();
+    assert(turb_model != NULL);
+    if (turb_model->itytur == 3)
+      t_visc_durb = t_visc / (kinetic_en * cs_turb_cmu ) * rnnb * 0.22;
+    else
+      t_visc_durb = t_visc;
 
-    *cofimp     = 1. - *ypup * (2.0 * sqrt( l_visc / t_visc_durb * dup1 * (1.0 - dup1) ) -  dup2) ;
+    *cofimp
+      = 1. - *ypup * (2. * sqrt(l_visc / t_visc_durb * dup1 * (1. - dup1))
+                      -  dup2);
 
     /* log layer */
     if (*yplus > ypluli) {
@@ -579,8 +595,8 @@ cs_wall_functions_2scales_scalable(cs_real_t   l_visc,
   *yplus = *uk * y / l_visc;
 
   /* Compute the friction velocity ustar */
-  *uk = cs_turb_cmu025 * sqrt(kinetic_en);
-  *yplus = *uk * y / l_visc;
+  *uk = cs_turb_cmu025 * sqrt(kinetic_en);//FIXME
+  *yplus = *uk * y / l_visc;//FIXME
 
   /* Log layer */
   if (*yplus > ypluli) {
@@ -593,7 +609,6 @@ cs_wall_functions_2scales_scalable(cs_real_t   l_visc,
   } else {
 
     *dplus = ypluli - *yplus;
-    *yplus = ypluli;
 
     /* Count the cell as if it was in the viscous sub-layer */
     *nsubla += 1;
@@ -601,13 +616,13 @@ cs_wall_functions_2scales_scalable(cs_real_t   l_visc,
   }
 
   /* Mixing length viscosity */
-  ml_visc = cs_turb_xkappa * l_visc * *yplus;
-  rcprod = CS_MIN(cs_turb_xkappa, CS_MAX(1., sqrt(ml_visc / t_visc)) / *yplus);
+  ml_visc = cs_turb_xkappa * l_visc * (*yplus + *dplus);
+  rcprod = CS_MIN(cs_turb_xkappa, CS_MAX(1., sqrt(ml_visc / t_visc)) / (*yplus + *dplus));
 
-  *ustar = vel / (log(*yplus) / cs_turb_xkappa + cs_turb_cstlog);
-  *ypup = (*yplus - *dplus) / (log(*yplus) / cs_turb_xkappa + cs_turb_cstlog);
+  *ustar = vel / (log(*yplus + *dplus) / cs_turb_xkappa + cs_turb_cstlog);
+  *ypup = *yplus / (log(*yplus + *dplus) / cs_turb_xkappa + cs_turb_cstlog);
   *cofimp = 1. - *ypup
-                 / cs_turb_xkappa * (2. * rcprod - 1. / (2. * *yplus - *dplus));
+                 / cs_turb_xkappa * (2. * rcprod - 1. / (2. * *yplus + *dplus));
 }
 
 /*----------------------------------------------------------------------------
@@ -802,15 +817,19 @@ cs_wall_functions_2scales_vdriest(cs_real_t   rnnb,
  *
  * \f$ u^+ \f$ is computed as follows:
  *   \f[ u^+ = \dfrac{1}{\kappa}
- *             \ln \left(\dfrac{(y+\xi) u_k}{\nu + \alpha \xi u_k} \right)
+ *             \ln \left(\dfrac{(y+y_0) u_k}{\nu + \alpha \xi u_k} \right)
  *            + Cst_{smooth} \f]
- * with \f$ \alpha = \exp \left(- \kappa(Cst_{rough}-Cst_{smooth})\right) \f$.
+ * with \f$ \alpha = \exp \left(- \kappa(Cst_{rough}-Cst_{smooth})\right)
+ *                 \simeq 0.26 \f$
+ * and \f$ y_0 = \alpha \xi \exp \left(-\kappa Cst_{smooth} \right)
+ *             = \xi \exp \left(-\kappa Cst_{rough} \right)
+ *             \simeq \dfrac{\xi}{33}\f$.
  *
  * \param[in]     l_visc        kinematic viscosity
  * \param[in]     t_visc        turbulent kinematic viscosity
  * \param[in]     vel           wall projected cell center velocity
  * \param[in]     y             wall distance
- * \param[in]     roughness     roughness
+ * \param[in]     rough_d       roughness length scale (not sand grain)
  * \param[in]     kinetic_en    turbulent kinetic energy
  * \param[out]    iuntur        indicator: 0 in the viscous sublayer
  * \param[out]    nsubla        counter of cell in the viscous sublayer
@@ -831,7 +850,7 @@ cs_wall_functions_2scales_smooth_rough(cs_real_t   l_visc,
                                        cs_real_t   t_visc,
                                        cs_real_t   vel,
                                        cs_real_t   y,
-                                       cs_real_t   roughness,
+                                       cs_real_t   rough_d,
                                        cs_real_t   kinetic_en,
                                        int        *iuntur,
                                        cs_lnum_t  *nsubla,
@@ -856,9 +875,17 @@ cs_wall_functions_2scales_smooth_rough(cs_real_t   l_visc,
    *
    * ln((y+y0)/y0) = ln((y+y0)/alpha xi) + kappa * 5.2
    *
-   * y0 =  roughness * exp(-kappa * 8.5)
-   * */
-  double y0 = roughness*exp(-cs_turb_xkappa*cs_turb_cstlog_rough);
+   * y0 =  xi * exp(-kappa * 8.5)
+   * where xi is the sand grain roughness here
+   * y0 = alpha * xi * exp(-kappa * 5.2)
+   *
+   * so:
+   *  alpha = exp(-kappa * (8.5 - 5.2)) = 0.25
+   *
+   */
+  cs_real_t y0 = rough_d;
+  /* Sand grain roughness */
+  cs_real_t sg_rough = rough_d * exp(cs_turb_xkappa*cs_turb_cstlog_rough);
 
   /* Blending for very low values of k */
   Re = sqrt(kinetic_en) * (y + y0) / l_visc;
@@ -867,40 +894,44 @@ cs_wall_functions_2scales_smooth_rough(cs_real_t   l_visc,
   *uk = sqrt( (1.-g) * cs_turb_cmu025 * cs_turb_cmu025 * kinetic_en
             + g * l_visc * vel / (y + y0));
 
-  double effective_visc = (l_visc + cs_turb_cstlog_alpha * roughness * *uk);
 
-  /* NB: tends to "y/xi" in rough regime, to "y.uk/nu" in smooth regime  */
-  *yplus = *uk * (y + y0) / effective_visc;
+  *yplus = *uk * y / l_visc;
 
-  double yk = *uk * y / l_visc;
+  /* As for scalable wall functions, yplus is shifted of "dplus" */
+  *dplus = *uk * y0 / l_visc;
 
-  /* As for scalable wall functions, "y*uk/effective_visc+ dplus = yplus" */
-  *dplus = *uk * y0 / effective_visc;
+  /* Shift of the velocity profile due to roughness */
+  cs_real_t shift_vel = -log(1. + cs_turb_cstlog_alpha * sg_rough * *uk/l_visc)
+    / cs_turb_xkappa;
 
   /* Log layer and shifted with the roughness */
-  if (yk > ypluli) {
+  if (*yplus > ypluli) {
 
     *nlogla += 1;
 
+  }
+
   /* Viscous sub-layer and therefore shift again */
-  } else {
+  else {
 
     *dplus = ypluli - *yplus;
-    *yplus = ypluli;
     /* Count the cell as if it was in the viscous sub-layer */
     *nsubla += 1;
 
   }
 
-  double uplus = log(*yplus) / cs_turb_xkappa + cs_turb_cstlog;
+  cs_real_t uplus = log(*yplus + *dplus) / cs_turb_xkappa + cs_turb_cstlog + shift_vel;
   *ustar = vel / uplus;
-  *ypup = yk / uplus;
+#if 0
+  bft_printf("uet=%f, u=%f, uplus=%f, yk=%f, vel=%f, duplus=%f\n", *ustar, vel, uplus, *yplus, vel, 1./uplus);
+#endif
+  *ypup = *yplus / uplus;
 
   /* Mixing length viscosity, compatible with both regimes */
   ml_visc = cs_turb_xkappa * *uk * (y + y0);
   rcprod = CS_MIN(cs_turb_xkappa, CS_MAX(1., sqrt(ml_visc / t_visc)) / *yplus);
-  *cofimp = 1. - (*yplus - *dplus) / uplus
-          / cs_turb_xkappa * ( 2. * rcprod - 1. / (2. * *yplus - *dplus));
+  *cofimp = 1. - *yplus / uplus
+          / cs_turb_xkappa * ( 2. * rcprod - 1. / (2. * *yplus + *dplus));
 
 }
 
@@ -987,19 +1018,26 @@ cs_wall_functions_disabled(cs_real_t   l_visc,
  *  h = \dfrac{K}{\centip \centf} h_{tur}
  *  \f]
  *
+ * \param[in]     l_visc        kinetic viscosity
  * \param[in]     prl           laminar Prandtl number
  * \param[in]     prt           turbulent Prandtl number
+ * \param[in]     rough_t       scalar roughness length scale
+ * \param[in]     uk            velocity scale based on TKE
  * \param[in]     yplus         dimensionless distance to the wall
  * \param[out]    dplus         dimensionless shift to the wall for scalable
  *                              wall functions
- * \param[out]    htur          corrected exchange coefficient
+ * \param[out]    htur          correction for the exchange coefficient
+ *                              (\f$ Pr y^+/T^+ \f$)
  * \param[out]    yplim         value of the limit for \f$ y^+ \f$
  */
 /*----------------------------------------------------------------------------*/
 
 inline static void
-cs_wall_functions_s_arpaci_larsen(cs_real_t  prl,
+cs_wall_functions_s_arpaci_larsen(cs_real_t  l_visc,
+                                  cs_real_t  prl,
                                   cs_real_t  prt,
+                                  cs_real_t  rough_t,
+                                  cs_real_t  uk,
                                   cs_real_t  yplus,
                                   cs_real_t  dplus,
                                   cs_real_t *htur,
@@ -1011,20 +1049,30 @@ cs_wall_functions_s_arpaci_larsen(cs_real_t  prl,
   double yp2;
   double prlm1;
 
-  const double epzero = 1.e-12;
-
-  /*==========================================================================*/
+  const double epzero = cs_math_epzero;
 
   /*==========================================================================
     1. Initializations
     ==========================================================================*/
 
-  /*==========================================================================*/
-
-  (*htur) = CS_MAX(yplus-dplus,epzero)/CS_MAX(yplus,epzero);
+  (*htur) = CS_MAX(yplus,epzero)/CS_MAX(yplus+dplus,epzero);
 
   prlm1 = 0.1;
 
+  /* Sand grain roughness is:
+   * zeta = z0 * exp(kappa 8.5)
+   * Then:
+   * hp = zeta uk / nu * exp( -kappa(8.5 - 5.2))
+   *    = z0 * uk / nu * exp(kappa * 5.2)
+   *   where 5.2 is the smooth log constant, and 8.5 the rough one
+   *
+   *   FIXME check if we should use a molecular Schmidt number
+   */
+  cs_real_t hp = rough_t *uk / l_visc * exp(cs_turb_xkappa*cs_turb_cstlog);
+
+  /* Shift of the temperature profile due to roughness */
+  cs_real_t shift_temp = -log(1. + hp);
+
   /*==========================================================================
     2. Compute htur for small Prandtl numbers
     ==========================================================================*/
@@ -1032,8 +1080,8 @@ cs_wall_functions_s_arpaci_larsen(cs_real_t  prl,
   if (prl <= prlm1) {
     (*yplim)   = prt/(prl*cs_turb_xkappa);
     if (yplus > (*yplim)) {
-      tplus = prl*(*yplim) + prt/cs_turb_xkappa * log(yplus/(*yplim));
-      (*htur) = prl*(yplus-dplus)/tplus;
+      tplus = prl*(*yplim) + prt/cs_turb_xkappa * (log((yplus+dplus)/(*yplim)) + shift_temp);
+      (*htur) = prl * yplus / tplus;
     }
 
     /*========================================================================
@@ -1041,21 +1089,20 @@ cs_wall_functions_s_arpaci_larsen(cs_real_t  prl,
       ========================================================================*/
 
   } else {
-    yp2   = cs_turb_xkappa*1000./prt;
-    yp2   = sqrt(yp2);
-    (*yplim)   = pow(1000./prl,1./3.);
+    yp2 = sqrt(cs_turb_xkappa*1000./prt);
+    (*yplim)   = pow(1000./prl, 1./3.);
 
-    a2 = 15.*pow(prl,2./3.);
-    beta2 = a2 - 500./ pow(yp2,2);
+    a2 = 15.*pow(prl, 2./3.);
 
     if (yplus >= (*yplim) && yplus < yp2) {
-      tplus = a2 - 500./(yplus*yplus);
-      (*htur) = prl*(yplus-dplus)/tplus;
+      tplus = a2 - 500./((yplus+dplus)*(yplus+dplus));
+      (*htur) = prl * yplus / tplus;
     }
 
     if (yplus >= yp2) {
-      tplus = beta2 + prt/cs_turb_xkappa*log(yplus/yp2);
-      (*htur) = prl*(yplus-dplus)/tplus;
+      beta2 = a2 - 0.5 * prt /cs_turb_xkappa;
+      tplus = beta2 + prt/cs_turb_xkappa*log((yplus+dplus)/yp2);
+      (*htur) = prl * yplus / tplus;
     }
 
   }
@@ -1082,7 +1129,8 @@ cs_wall_functions_s_arpaci_larsen(cs_real_t  prl,
  *                              ( \f$ Pr=\sigma=\frac{\mu C_p}{\lambda_f} \f$ )
  * \param[in]     prt           turbulent Prandtl number ( \f$ \sigma_t \f$ )
  * \param[in]     yplus         dimensionless distance to the wall
- * \param[out]    htur          corrected exchange coefficient
+ * \param[out]    htur          correction for the exchange coefficient
+ *                              (\f$ Pr y^+/T^+ \f$)
  */
 /*----------------------------------------------------------------------------*/
 
@@ -1133,6 +1181,60 @@ cs_wall_functions_s_vdriest(cs_real_t  prl,
     }
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ *  \brief Rough Smooth Thermal Wall Function - Prototype
+ *
+ * \param[in]     l_visc        kinetic viscosity
+ * \param[in]     prl           molecular Prandtl number
+ *                              ( \f$ Pr=\sigma=\frac{\mu C_p}{\lambda_f} \f$ )
+ * \param[in]     prt           turbulent Prandtl number ( \f$ \sigma_t \f$ )
+ * \param[in]     rough_t       scalar roughness length scale
+ * \param[in]     uk            velocity scale based on TKE
+ * \param[in]     yplus         dimensionless distance to the wall
+ * \param[in]     dplus         dimensionless shift to the wall for scalable
+ *                              wall function
+ * \param[out]    htur          correction for the exchange coefficient
+ *                              (\f$ Pr y^+/T^+ \f$)
+ */
+/*----------------------------------------------------------------------------*/
+
+inline static void
+cs_wall_functions_s_smooth_rough(cs_real_t  l_visc,
+                                 cs_real_t  prl,
+                                 cs_real_t  prt,
+                                 cs_real_t  rough_t,
+                                 cs_real_t  uk,
+                                 cs_real_t  yplus,
+                                 cs_real_t  dplus,
+                                 cs_real_t *htur)
+{
+  CS_UNUSED(prt);
+
+  /* Sand grain roughness is:
+   * zeta = z0 * exp(kappa 8.5)
+   * Then:
+   * hp = zeta uk / nu * exp( -kappa(8.5 - 5.2))
+   *    = z0 * uk / nu * exp(kappa * 5.2)
+   *   where 5.2 is the smooth log constant, and 8.5 the rough one
+   *
+   *   FIXME check if we should use a molecular Schmidt number
+   */
+  cs_real_t hp = rough_t *uk / l_visc * exp(cs_turb_xkappa*cs_turb_cstlog);
+  const double ypluli = cs_glob_wall_functions->ypluli;
+  const double epzero = cs_math_epzero;
+
+  (*htur) = CS_MAX(yplus,epzero)/CS_MAX(yplus+dplus,epzero);
+
+  /* Shift of the temperature profile due to roughness */
+  cs_real_t shift_temp = -log(1. + hp);
+
+  if (yplus > ypluli) {
+    cs_real_t tplus = prt * ((log(yplus+dplus) + shift_temp)/cs_turb_xkappa + cs_turb_cstlog);
+    (*htur) = prl * yplus / tplus;
+  }
+}
+
 /*============================================================================
  * Public function definitions for Fortran API
  *============================================================================*/
@@ -1143,16 +1245,16 @@ cs_wall_functions_s_vdriest(cs_real_t  prl,
 
 void CS_PROCF (wallfunctions, WALLFUNCTIONS)
 (
- const cs_int_t   *const iwallf,
+ const int        *const iwallf,
  const cs_lnum_t  *const ifac,
  const cs_real_t  *const viscosity,
  const cs_real_t  *const t_visc,
  const cs_real_t  *const vel,
  const cs_real_t  *const y,
- const cs_real_t  *const roughness,
+ const cs_real_t  *const rough_d,
  const cs_real_t  *const rnnb,
  const cs_real_t  *const kinetic_en,
-       cs_int_t         *iuntur,
+       int              *iuntur,
        cs_lnum_t        *nsubla,
        cs_lnum_t        *nlogla,
        cs_real_t        *ustar,
@@ -1169,9 +1271,12 @@ void CS_PROCF (wallfunctions, WALLFUNCTIONS)
 
 void CS_PROCF (hturbp, HTURBP)
 (
- const cs_int_t   *const iwalfs,
+ const int        *const iwalfs,
+ const cs_real_t  *const l_visc,
  const cs_real_t  *const prl,
  const cs_real_t  *const prt,
+ const cs_real_t  *const rough_t,
+ const cs_real_t  *const uk,
  const cs_real_t  *const yplus,
  const cs_real_t  *const dplus,
        cs_real_t        *htur,
@@ -1183,23 +1288,16 @@ void CS_PROCF (hturbp, HTURBP)
  *============================================================================*/
 
 /*----------------------------------------------------------------------------
- *! \brief Provide acces to cs_glob_wall_functions
- *
- * needed to initialize structure with GUI
+ *! \brief Provide access to cs_glob_wall_functions
  *----------------------------------------------------------------------------*/
 
 cs_wall_functions_t *
 cs_get_glob_wall_functions(void);
 
-/*! \brief  Compute the friction velocity and \f$y^+\f$ / \f$u^+\f$.
-
-*/
-/*-------------------------------------------------------------------------------
-  Arguments
- ______________________________________________________________________________.
-   mode           name          role                                           !
- ______________________________________________________________________________*/
+/*----------------------------------------------------------------------------*/
 /*!
+ * \brief  Compute the friction velocity and \f$y^+\f$ / \f$u^+\f$.
+ *
  * \param[in]     iwallf        wall function type
  * \param[in]     ifac          face number
  * \param[in]     l_visc        kinematic viscosity
@@ -1207,7 +1305,7 @@ cs_get_glob_wall_functions(void);
  * \param[in]     vel           wall projected
  *                              cell center velocity
  * \param[in]     y             wall distance
- * \param[in]     roughness     roughness
+ * \param[in]     rough_d       roughness lenghth scale
  * \param[in]     rnnb          \f$\vec{n}.(\tens{R}\vec{n})\f$
  * \param[in]     kinetic_en    turbulent kinetic energy
  * \param[in]     iuntur        indicator:
@@ -1224,7 +1322,7 @@ cs_get_glob_wall_functions(void);
  * \param[out]    dplus         dimensionless shift to the wall
  *                              for scalable wall functions
  */
-/*-------------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------------*/
 
 void
 cs_wall_functions_velocity(cs_wall_f_type_t  iwallf,
@@ -1233,7 +1331,7 @@ cs_wall_functions_velocity(cs_wall_f_type_t  iwallf,
                            cs_real_t         t_visc,
                            cs_real_t         vel,
                            cs_real_t         y,
-                           cs_real_t         roughness,
+                           cs_real_t         rough_d,
                            cs_real_t         rnnb,
                            cs_real_t         kinetic_en,
                            int              *iuntur,
@@ -1246,11 +1344,10 @@ cs_wall_functions_velocity(cs_wall_f_type_t  iwallf,
                            cs_real_t        *cofimp,
                            cs_real_t        *dplus);
 
-/*-------------------------------------------------------------------------------*/
-
+/*----------------------------------------------------------------------------*/
 /*!
- *  \brief Compute the correction of the exchange coefficient between the fluid and
- *  the wall for a turbulent flow.
+ *  \brief Compute the correction of the exchange coefficient between the
+ *         fluid and the wall for a turbulent flow.
  *
  *  This is function of the dimensionless
  *  distance to the wall \f$ y^+ = \dfrac{\centip \centf u_\star}{\nu}\f$.
@@ -1260,28 +1357,27 @@ cs_wall_functions_velocity(cs_wall_f_type_t  iwallf,
  *  h_{tur} = Pr \dfrac{y^+}{T^+}
  *  \f]
  *
- */
-/*-------------------------------------------------------------------------------
-  Arguments
- ______________________________________________________________________________.
-   mode           name          role                                           !
- ______________________________________________________________________________*/
-/*!
  * \param[in]     iwalfs        type of wall functions for scalar
+ * \param[in]     l_visc        kinematic viscosity
  * \param[in]     prl           laminar Prandtl number
  * \param[in]     prt           turbulent Prandtl number
+ * \param[in]     rough_t       scalar roughness lenghth scale
+ * \param[in]     uk            velocity scale based on TKE
  * \param[in]     yplus         dimensionless distance to the wall
  * \param[in]     dplus         dimensionless distance for scalable
  *                              wall functions
  * \param[out]    htur          corrected exchange coefficient
  * \param[out]    yplim         value of the limit for \f$ y^+ \f$
  */
-/*-------------------------------------------------------------------------------*/
+/*----------------------------------------------------------------------------*/
 
 void
 cs_wall_functions_scalar(cs_wall_f_s_type_t  iwalfs,
+                         cs_real_t           l_visc,
                          cs_real_t           prl,
                          cs_real_t           prt,
+                         cs_real_t           rough_t,
+                         cs_real_t           uk,
                          cs_real_t           yplus,
                          cs_real_t           dplus,
                          cs_real_t          *htur,
