@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2018 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -63,7 +63,6 @@
 #include "cs_order.h"
 #include "cs_parall.h"
 #include "cs_physical_model.h"
-#include "cs_prototypes.h"
 #include "cs_random.h"
 #include "cs_search.h"
 #include "cs_timer_stats.h"
@@ -72,10 +71,10 @@
 #include "cs_field_pointer.h"
 
 #include "cs_lagr_clogging.h"
+#include "cs_lagr_deposition_model.h"
 #include "cs_lagr_roughness.h"
 #include "cs_lagr_dlvo.h"
 #include "cs_lagr_stat.h"
-#include "cs_lagr_geom.h"
 #include "cs_lagr.h"
 #include "cs_lagr_tracking.h"
 
@@ -296,7 +295,8 @@ _random_point_in_face(cs_lnum_t        n_vertices,
  * \param[in]      face_ids           ids of faces in zone
  * \param[in]      face_particle_idx  starting index of added particles
  *                                    for each face in zone
- *----------------------------------------------------------------------------*/
+ */
+/*----------------------------------------------------------------------------*/
 
 void
 cs_lagr_new(cs_lagr_particle_set_t  *particles,
@@ -351,7 +351,7 @@ cs_lagr_new(cs_lagr_particle_set_t  *particles,
 
       cs_lnum_t p_id = p_s_id + i;
 
-      cs_lagr_particles_set_lnum(particles, p_id, CS_LAGR_CELL_NUM, c_id+1);
+      cs_lagr_particles_set_lnum(particles, p_id, CS_LAGR_CELL_ID, c_id);
 
       cs_real_t *part_coord
         = cs_lagr_particles_attr(particles, p_id, CS_LAGR_COORDS);
@@ -363,7 +363,7 @@ cs_lagr_new(cs_lagr_particle_set_t  *particles,
                             acc_surf_r,
                             part_coord);
 
-      /* For safety, move particle slighty inside cell */
+      /* For safety, move particle slightly inside cell */
 
       for (cs_lnum_t j = 0; j < 3; j++)
         part_coord[j] += (c_cen[j] - part_coord[j])*d_eps;
@@ -388,7 +388,8 @@ cs_lagr_new(cs_lagr_particle_set_t  *particles,
  * \param[in]      cell_ids           ids of cells in zone
  * \param[in]      cell_particle_idx  starting index of added particles
  *                                    for each cell in zone
- *----------------------------------------------------------------------------*/
+ */
+/*----------------------------------------------------------------------------*/
 
 void
 cs_lagr_new_v(cs_lagr_particle_set_t  *particles,
@@ -540,7 +541,7 @@ cs_lagr_new_v(cs_lagr_particle_set_t  *particles,
 
       cs_lnum_t p_id = p_s_id + i;
 
-      cs_lagr_particles_set_lnum(particles, p_id, CS_LAGR_CELL_NUM, cell_id+1);
+      cs_lagr_particles_set_lnum(particles, p_id, CS_LAGR_CELL_ID, cell_id);
 
       cs_real_t *part_coord
         = cs_lagr_particles_attr(particles, p_id, CS_LAGR_COORDS);
@@ -601,7 +602,7 @@ cs_lagr_new_v(cs_lagr_particle_set_t  *particles,
           part_coord[j] += (cell_cen[j] - part_coord[j]) * (1. - t);
       }
 
-      /* Move particle slighty towards cell center cell
+      /* Move particle slightly towards cell center cell
          (assuming cell is star-shaped) */
 
       else {
@@ -646,6 +647,7 @@ cs_lagr_new_particle_init(const cs_lnum_t  particle_range[2],
 
   cs_lagr_extra_module_t  *extra = cs_get_lagr_extra_module();
 
+  cs_lnum_t n_cells = cs_glob_mesh->n_cells;
   /* Map field arrays */
 
   const cs_real_3_t  *vel = NULL;
@@ -663,7 +665,7 @@ cs_lagr_new_particle_init(const cs_lnum_t  particle_range[2],
      as seen by solid particles along their trajectory.
      -------------------------------------------------- */
 
-  if (cs_glob_lagr_time_scheme->idistu == 1) {
+  if (cs_glob_lagr_model->idistu == 1) {
 
     if (extra->cvar_k != NULL)
       cvar_k = (const cs_real_t *)extra->cvar_k->vals[time_id];
@@ -671,6 +673,7 @@ cs_lagr_new_particle_init(const cs_lnum_t  particle_range[2],
     else if (extra->cvar_rij != NULL)
       cvar_rij = (const cs_real_6_t *) extra->cvar_rij->vals[time_id];
 
+    /* Deprecated irijco = 0 */
     else if (extra->cvar_r11 != NULL) {
       cvar_r11 = (const cs_real_t *)extra->cvar_r11->vals[time_id];
       cvar_r22 = (const cs_real_t *)extra->cvar_r22->vals[time_id];
@@ -683,11 +686,11 @@ cs_lagr_new_particle_init(const cs_lnum_t  particle_range[2],
          _("The Lagrangian module is incompatible with the selected\n"
            " turbulence model.\n\n"
            "Turbulent dispersion is used with:\n"
-           "  cs_glob_lagr_time_scheme->idistu = %d\n"
+           "  cs_glob_lagr_model->idistu = %d\n"
            "And the turbulence model is iturb = %d\n\n"
            "The only turbulence models compatible with the Lagrangian model's\n"
            "turbulent dispersion are k-epsilon, Rij-epsilon, v2f, and k-omega."),
-         cs_glob_lagr_time_scheme->idistu,
+         cs_glob_lagr_model->idistu,
          extra->iturb);
     }
 
@@ -700,7 +703,7 @@ cs_lagr_new_particle_init(const cs_lnum_t  particle_range[2],
 
   BFT_MALLOC(vagaus, n, cs_real_3_t);
 
-  if (cs_glob_lagr_time_scheme->idistu == 1 && n > 0) {
+  if (cs_glob_lagr_model->idistu == 1 && n > 0) {
     cs_random_normal(n*3, (cs_real_t *)vagaus);
   }
 
@@ -714,41 +717,113 @@ cs_lagr_new_particle_init(const cs_lnum_t  particle_range[2],
 
   }
 
+  cs_real_33_t *eig_vec;
+  cs_real_3_t *eig_val;
+
+  BFT_MALLOC(eig_vec, n_cells, cs_real_33_t);
+  BFT_MALLOC(eig_val, n_cells, cs_real_3_t);
+
+  /* Initialisation from the mean eulerian fluid */
+  if (cs_glob_lagr_model->idistu == 1) {
+
+    cs_real_33_t *sym_rij;
+    BFT_MALLOC(sym_rij, n_cells, cs_real_33_t);
+    cs_real_t tol_err = 1.0e-12;
+
+    for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
+      if (cvar_rij != NULL) {
+        sym_rij[cell_id][0][0] = cvar_rij[cell_id][0];
+        sym_rij[cell_id][1][1] = cvar_rij[cell_id][1];
+        sym_rij[cell_id][2][2] = cvar_rij[cell_id][2];
+        sym_rij[cell_id][0][1] = cvar_rij[cell_id][3];
+        sym_rij[cell_id][1][0] = cvar_rij[cell_id][3];
+        sym_rij[cell_id][1][2] = cvar_rij[cell_id][4];
+        sym_rij[cell_id][2][1] = cvar_rij[cell_id][4];
+        sym_rij[cell_id][0][2] = cvar_rij[cell_id][5];
+        sym_rij[cell_id][2][0] = cvar_rij[cell_id][5];
+      }
+      /* TODO do it better for EVM models */
+      else {
+        cs_real_t w = 0.;
+
+        if (cvar_k != NULL)
+          w = d2s3 * cvar_k[cell_id];
+        /* Deprecated irijco = 0 */
+        else if (cvar_r11 != NULL)
+          w = (cvar_r11[cell_id] + cvar_r22[cell_id] + cvar_r33[cell_id]) / 3.;
+
+        sym_rij[cell_id][0][0] = w;
+        sym_rij[cell_id][1][1] = w;
+        sym_rij[cell_id][2][2] = w;
+        sym_rij[cell_id][0][1] = 0.;
+        sym_rij[cell_id][1][0] = 0.;
+        sym_rij[cell_id][1][2] = 0.;
+        sym_rij[cell_id][2][1] = 0.;
+        sym_rij[cell_id][0][2] = 0.;
+        sym_rij[cell_id][2][0] = 0.;
+      }
+
+      eig_vec[cell_id][0][0] = 1;
+      eig_vec[cell_id][0][1] = 0;
+      eig_vec[cell_id][0][2] = 0;
+      eig_vec[cell_id][1][0] = 0;
+      eig_vec[cell_id][1][1] = 1;
+      eig_vec[cell_id][1][2] = 0;
+      eig_vec[cell_id][2][0] = 0;
+      eig_vec[cell_id][2][1] = 0;
+      eig_vec[cell_id][2][2] = 1;
+
+      cs_math_33_eig_val_vec(sym_rij[cell_id], tol_err, eig_val[cell_id], eig_vec[cell_id]);
+    }
+
+    BFT_FREE(sym_rij);
+  } else {
+    for (cs_lnum_t cell_id = 0; cell_id < n_cells; cell_id++) {
+
+      eig_vec[cell_id][0][0] = 1.;
+      eig_vec[cell_id][1][1] = 1.;
+      eig_vec[cell_id][2][2] = 1.;
+      eig_vec[cell_id][0][1] = 0.;
+      eig_vec[cell_id][0][2] = 0.;
+      eig_vec[cell_id][1][0] = 0.;
+      eig_vec[cell_id][1][2] = 0.;
+      eig_vec[cell_id][2][0] = 0.;
+      eig_vec[cell_id][2][1] = 0.;
+      eig_val[cell_id][0] = 0.;
+      eig_val[cell_id][1] = 0.;
+      eig_val[cell_id][2] = 0.;
+    }
+  }
+
   for (cs_lnum_t p_id = particle_range[0]; p_id < particle_range[1]; p_id++) {
 
     unsigned char *particle = pset->p_buffer + p_am->extents * p_id;
 
-    cs_lnum_t iel  = cs_lagr_particle_get_cell_id(particle, p_am);
+    cs_lnum_t cell_id  = cs_lagr_particle_get_lnum(particle, p_am, CS_LAGR_CELL_ID);
     cs_lnum_t l_id = p_id - particle_range[0];
 
     cs_real_t  *vel_seen
       = cs_lagr_particle_attr(particle, p_am, CS_LAGR_VELOCITY_SEEN);
 
-    cs_real_t w = 0.;
-
-    if (cs_glob_lagr_time_scheme->idistu == 1) {
-      if (cvar_k != NULL)
-        w = cvar_k[iel];
-      else if (cvar_rij != NULL)
-        w = 0.5*(cvar_rij[iel][0] + cvar_rij[iel][1] + cvar_rij[iel][2]);
-      else if (cvar_r11 != NULL)
-        w = 0.5 * (cvar_r11[iel] + cvar_r22[iel] + cvar_r33[iel]);
+    for (cs_lnum_t i = 0; i < 3; i++) {
+      vel_seen[i] = vel[cell_id][i]
+                  + vagaus[l_id][0] * sqrt(eig_val[cell_id][0]) * eig_vec[cell_id][0][i]
+                  + vagaus[l_id][1] * sqrt(eig_val[cell_id][1]) * eig_vec[cell_id][1][i]
+                  + vagaus[l_id][2] * sqrt(eig_val[cell_id][2]) * eig_vec[cell_id][2][i];
     }
 
-    cs_real_t tu = sqrt(d2s3 * w);
-
-    for (cs_lnum_t i = 0; i < 3; i++)
-      vel_seen[i] = vel[iel][i] + vagaus[l_id][i] * tu;
+    cs_lagr_particle_set_lnum(particle, p_am, CS_LAGR_P_FLAG, 0);
 
     cs_lagr_particle_set_lnum(particle, p_am, CS_LAGR_REBOUND_ID, -1);
-
     cs_lagr_particle_set_real(particle, p_am, CS_LAGR_TR_TRUNCATE, 0);
 
   }
 
   BFT_FREE(vagaus);
+  BFT_FREE(eig_vec);
+  BFT_FREE(eig_val);
 
-  /* Compute velcocity fluctuation if deposition model is active */
+  /* Compute velocity fluctuation if deposition model is active */
 
   if (cs_glob_lagr_model->deposition == 1) {
 
@@ -758,7 +833,8 @@ cs_lagr_new_particle_init(const cs_lnum_t  particle_range[2],
 
       unsigned char *particle = pset->p_buffer + p_am->extents * p_id;
 
-      cs_lnum_t iel = cs_lagr_particle_get_cell_id(particle, p_am);
+      cs_lnum_t iel  = cs_lagr_particle_get_lnum(particle, p_am,
+                                                 CS_LAGR_CELL_ID);
 
       /* Compute normalized wall-normal particle distance (y+) */
 
@@ -785,33 +861,37 @@ cs_lagr_new_particle_init(const cs_lnum_t  particle_range[2],
           cs_lnum_t  *neighbor_face_id;
           cs_real_t  *particle_yplus;
 
-          if (cs_glob_lagr_model->deposition > 0) {
-            neighbor_face_id
-              = cs_lagr_particle_attr(particle, p_am, CS_LAGR_NEIGHBOR_FACE_ID);
-            particle_yplus
-              = cs_lagr_particle_attr(particle, p_am, CS_LAGR_YPLUS);
-          }
-          else {
-            neighbor_face_id = NULL;
-            particle_yplus = 0;  /* allow tests even without particle y+ */
-          }
+          neighbor_face_id
+            = cs_lagr_particle_attr(particle, p_am, CS_LAGR_NEIGHBOR_FACE_ID);
+          particle_yplus
+            = cs_lagr_particle_attr(particle, p_am, CS_LAGR_YPLUS);
 
           cs_lagr_test_wall_cell(particle, p_am, visc_length,
                                  particle_yplus, neighbor_face_id);
 
         }
+        else {
+          cs_lagr_particle_set_lnum(particle, p_am, CS_LAGR_NEIGHBOR_FACE_ID, -1);
+          cs_lagr_particle_set_real(particle, p_am, CS_LAGR_YPLUS, 0.);
+        }
 
       }
 
       if (yplus < cs_lagr_particle_get_real(particle, p_am, CS_LAGR_INTERF)) {
 
-        cs_lagr_particle_set_lnum(particle, p_am, CS_LAGR_MARKO_VALUE, 10);
+        cs_lagr_particle_set_lnum(particle,
+                                  p_am,
+                                  CS_LAGR_MARKO_VALUE,
+                                  CS_LAGR_COHERENCE_STRUCT_DEGEN_INNER_ZONE_DIFF);
 
       }
 
       else if (yplus > 100.0) {
 
-        cs_lagr_particle_set_lnum(particle, p_am, CS_LAGR_MARKO_VALUE, -1);
+        cs_lagr_particle_set_lnum(particle,
+                                  p_am,
+                                  CS_LAGR_MARKO_VALUE,
+                                  CS_LAGR_COHERENCE_STRUCT_BULK);
 
       }
 
@@ -821,13 +901,22 @@ cs_lagr_new_particle_init(const cs_lnum_t  particle_range[2],
         cs_random_uniform(1, &random);
 
         if (random < 0.25)
-          cs_lagr_particle_set_lnum(particle, p_am, CS_LAGR_MARKO_VALUE, 12);
+          cs_lagr_particle_set_lnum(particle,
+                                    p_am,
+                                    CS_LAGR_MARKO_VALUE,
+                                    CS_LAGR_COHERENCE_STRUCT_DEGEN_DIFFUSION);
 
         else if (random > 0.625)
-          cs_lagr_particle_set_lnum(particle, p_am, CS_LAGR_MARKO_VALUE, 1);
+          cs_lagr_particle_set_lnum(particle,
+                                    p_am,
+                                    CS_LAGR_MARKO_VALUE,
+                                    CS_LAGR_COHERENCE_STRUCT_SWEEP);
 
         else /* if ((random > 0.25) && (random < 0.625)) */
-          cs_lagr_particle_set_lnum(particle, p_am, CS_LAGR_MARKO_VALUE, 3);
+          cs_lagr_particle_set_lnum(particle,
+                                    p_am,
+                                    CS_LAGR_MARKO_VALUE,
+                                    CS_LAGR_COHERENCE_STRUCT_EJECTION);
 
       }
 
@@ -841,10 +930,6 @@ cs_lagr_new_particle_init(const cs_lnum_t  particle_range[2],
 
       }
 
-      /* No deposited particles at the injection */
-      cs_lagr_particle_set_lnum(particle, p_am, CS_LAGR_DEPOSITION_FLAG,
-                                CS_LAGR_PART_IN_FLOW);
-
       /* Initialization of additional "pointers"
        * for the resuspension model              */
 
