@@ -5,7 +5,7 @@
 /*
   This file is part of Code_Saturne, a general-purpose CFD tool.
 
-  Copyright (C) 1998-2016 EDF S.A.
+  Copyright (C) 1998-2021 EDF S.A.
 
   This program is free software; you can redistribute it and/or modify it under
   the terms of the GNU General Public License as published by the Free Software
@@ -44,6 +44,7 @@
 
 #include "cs_base.h"
 #include "cs_mesh.h"
+#include "cs_sort.h"
 
 #include "fvm_defs.h"
 #include "fvm_nodal.h"
@@ -90,19 +91,19 @@ BEGIN_C_DECLS
  * parameters:
  *   mesh             <-- base mesh
  *   extr_mesh        <-- extracted mesh name
+ *   boundary_flag    <-- -1 if unspecified, 0 if faces are not on boundary,
+ *                        1 if faces are on boundary
  *   include_families <-- include family info if true
  *   i_face_list_size <-- size of i_face_list[] array
  *   b_face_list_size <-- size of b_face_list[] array
  *   i_face_list      <-> list of interior faces (1 to n), or NULL
  *   b_face_list      <-> list of boundary faces (1 to n), or NULL
- *
- * returns:
- *   pointer to extracted nodal mesh
  *----------------------------------------------------------------------------*/
 
 static void
 _add_faces_to_nodal(const cs_mesh_t  *mesh,
                     fvm_nodal_t      *extr_mesh,
+                    int               boundary_flag,
                     bool              include_families,
                     cs_lnum_t         i_face_list_size,
                     cs_lnum_t         b_face_list_size,
@@ -124,10 +125,6 @@ _add_faces_to_nodal(const cs_mesh_t  *mesh,
   const int   *_face_families[2];
   const int   **face_families = NULL;
 
-  cs_gnum_t  *num_glob_fac = NULL;
-
-  assert(sizeof(int) == sizeof(cs_lnum_t)); /* For families */
-
   /* Count the number of faces to convert */
 
   n_max_faces = mesh->n_i_faces + mesh->n_b_faces;
@@ -230,6 +227,7 @@ _add_faces_to_nodal(const cs_mesh_t  *mesh,
   face_vertices_num[1] = mesh->i_face_vtx_lst;
 
   fvm_nodal_from_desc_add_faces(extr_mesh,
+                                boundary_flag,
                                 extr_face_count,
                                 extr_face_list,
                                 2,
@@ -240,6 +238,29 @@ _add_faces_to_nodal(const cs_mesh_t  *mesh,
                                 NULL);
 
   BFT_FREE(extr_face_list);
+}
+
+/*----------------------------------------------------------------------------
+ * Order added faces in an external mesh.
+ *
+ * parameters:
+ *   mesh             <-- base mesh
+ *   extr_mesh        <-- extracted mesh name
+ *----------------------------------------------------------------------------*/
+
+static void
+_order_nodal_faces(const cs_mesh_t  *mesh,
+                   fvm_nodal_t      *extr_mesh)
+{
+  cs_lnum_t   face_id, i;
+
+  cs_lnum_t   n_max_faces = 0;
+
+  cs_gnum_t  *num_glob_fac = NULL;
+
+  /* Count the number of faces to convert */
+
+  n_max_faces = mesh->n_i_faces + mesh->n_b_faces;
 
   /* In case of parallelism or face renumbering, sort faces by
      increasing global number */
@@ -328,7 +349,7 @@ cs_mesh_connect_get_cell_faces(const cs_mesh_t         *mesh,
   if (extr_cell_id != NULL)
     n_loc_cells = extr_cell_size;
 
-  BFT_MALLOC(cell_faces_idx, n_loc_cells + 1, cs_int_t);
+  BFT_MALLOC(cell_faces_idx, n_loc_cells + 1, cs_lnum_t);
 
   for (cell_id = 0; cell_id < n_loc_cells + 1; cell_id++)
     cell_faces_idx[cell_id] = 0;
@@ -375,8 +396,8 @@ cs_mesh_connect_get_cell_faces(const cs_mesh_t         *mesh,
 
   /* Build array of values */
 
-  BFT_MALLOC(cell_faces_val, cell_faces_idx[n_loc_cells] - 1, cs_int_t);
-  BFT_MALLOC(cell_face_count, n_loc_cells, cs_int_t);
+  BFT_MALLOC(cell_faces_val, cell_faces_idx[n_loc_cells] - 1, cs_lnum_t);
+  BFT_MALLOC(cell_face_count, n_loc_cells, cs_lnum_t);
 
   for (cell_id = 0; cell_id < n_loc_cells; cell_id++)
     cell_face_count[cell_id] = 0;
@@ -426,7 +447,7 @@ cs_mesh_connect_get_cell_faces(const cs_mesh_t         *mesh,
 
 #if 0 && defined(DEBUG) && !defined(NDEBUG)
  {
-   cs_int_t ipos, ival;
+   cs_lnum_t ipos, ival;
    /* Print arrays */
    bft_printf("dbg : cs_mesh_ret_cel_fac\n"
               "nombre de cellules extraites = %d\n", extr_cell_size);
@@ -618,14 +639,13 @@ cs_mesh_connect_cells_to_nodal(const cs_mesh_t  *mesh,
 
   extr_mesh = fvm_nodal_create(name, 3);
 
-  assert(sizeof(int) == sizeof(cs_lnum_t)); /* For families */
+  fvm_nodal_set_parent(extr_mesh, mesh);
 
   if (include_families)
     cell_family = mesh->cell_family;
 
   fvm_nodal_from_desc_add_cells(extr_mesh,
                                 extr_cell_count,
-                                NULL,
                                 2,
                                 face_num_shift,
                                 (const cs_lnum_t **)face_vertices_idx,
@@ -642,11 +662,23 @@ cs_mesh_connect_cells_to_nodal(const cs_mesh_t  *mesh,
 
     _add_faces_to_nodal(mesh,
                         extr_mesh,
+                        1,
+                        include_families,
+                        0,
+                        b_face_count,
+                        NULL,
+                        b_face_list);
+
+    _add_faces_to_nodal(mesh,
+                        extr_mesh,
+                        0,
                         include_families,
                         i_face_count,
-                        b_face_count,
+                        0,
                         i_face_list,
-                        b_face_list);
+                        NULL);
+
+    _order_nodal_faces(mesh, extr_mesh);
 
     BFT_FREE(i_face_list);
     BFT_FREE(b_face_list);
@@ -710,8 +742,6 @@ cs_mesh_connect_faces_to_nodal(const cs_mesh_t  *mesh,
 {
   fvm_nodal_t  *extr_mesh = NULL;
 
-  assert(sizeof(int) == sizeof(cs_lnum_t)); /* For families */
-
   /* Check that the mesh contains face -> vertices connectivity */
 
   if (mesh->b_face_vtx_idx == NULL || mesh->i_face_vtx_idx == NULL)
@@ -722,14 +752,19 @@ cs_mesh_connect_faces_to_nodal(const cs_mesh_t  *mesh,
 
   extr_mesh = fvm_nodal_create(name, 3);
 
+  fvm_nodal_set_parent(extr_mesh, mesh);
+
   _add_faces_to_nodal(mesh,
                       extr_mesh,
+                      -1,
                       include_families,
                       i_face_list_size,
                       b_face_list_size,
                       i_face_list,
                       b_face_list);
 
+  _order_nodal_faces(mesh, extr_mesh);
+
   fvm_nodal_set_shared_vertices(extr_mesh,
                                 mesh->vtx_coord);
 
@@ -748,6 +783,150 @@ cs_mesh_connect_faces_to_nodal(const cs_mesh_t  *mesh,
   return extr_mesh;
 }
 
+/*----------------------------------------------------------------------------*/
+/*!
+ * \brief Build a vertex to cell connectivity for marked vertices only.
+ *
+ * It is the caller's responsibility to free the v2c_idx and v2c arrays,
+ * which are allocated by this function.
+ *
+ * \param[in]    mesh      pointer to mesh structure
+ * \param[in]    v_flag    vertex selection flag (0: not selected, 1: selected)
+ * \param[out]   v2c_idx   vertex to cells index (size: mesh->n_vertices +1)
+ * \param[out]   v2c       vertex to cells
+ */
+/*----------------------------------------------------------------------------*/
+
+void
+cs_mesh_connect_vertices_to_cells(cs_mesh_t    *mesh,
+                                  const char    v_flag[],
+                                  cs_lnum_t   **v2c_idx,
+                                  cs_lnum_t   **v2c)
+{
+  cs_lnum_t n_vertices = mesh->n_vertices;
+
+  /* Mark vertices which may be split (vertices lying on new boundary faces) */
+
+  cs_lnum_t  *_v2c_idx;
+  BFT_MALLOC(_v2c_idx, n_vertices+1, cs_lnum_t);
+
+  _v2c_idx[0] = 0;
+  for (cs_lnum_t i = 0; i < n_vertices; i++)
+    _v2c_idx[i+1] = 0;
+
+  /* Now build vertex -> cells index
+     (which will contain duplicate entries at first) */
+
+  for (cs_lnum_t f_id = 0; f_id < mesh->n_i_faces; f_id++) {
+    cs_lnum_t s_id = mesh->i_face_vtx_idx[f_id];
+    cs_lnum_t e_id = mesh->i_face_vtx_idx[f_id+1];
+    for (cs_lnum_t i = s_id; i < e_id; i++) {
+      cs_lnum_t vtx_id = mesh->i_face_vtx_lst[i];
+      if (v_flag[vtx_id] != 0) {
+        if (mesh->i_face_cells[f_id][0] > -1)
+          _v2c_idx[vtx_id + 1] += 1;
+        if (mesh->i_face_cells[f_id][1] > -1)
+          _v2c_idx[vtx_id + 1] += 1;
+      }
+    }
+  }
+
+  for (cs_lnum_t f_id = 0; f_id < mesh->n_b_faces; f_id++) {
+    cs_lnum_t s_id = mesh->b_face_vtx_idx[f_id];
+    cs_lnum_t e_id = mesh->b_face_vtx_idx[f_id+1];
+    for (cs_lnum_t i = s_id; i < e_id; i++) {
+      cs_lnum_t vtx_id = mesh->b_face_vtx_lst[i];
+      if (v_flag[vtx_id] != 0)
+        _v2c_idx[vtx_id + 1] += 1;
+    }
+  }
+
+  /* Transform count to index */
+
+  for (cs_lnum_t i = 0; i < n_vertices; i++)
+    _v2c_idx[i+1] += _v2c_idx[i];
+
+  /* Now define selected vertex->cell adjacency */
+
+  cs_lnum_t *_v2c;
+  BFT_MALLOC(_v2c, _v2c_idx[n_vertices], cs_lnum_t);
+
+  cs_lnum_t *v2c_count;
+  BFT_MALLOC(v2c_count, n_vertices, cs_lnum_t);
+  for (cs_lnum_t i = 0; i < n_vertices; i++)
+    v2c_count[i] = 0;
+
+  for (cs_lnum_t f_id = 0; f_id < mesh->n_i_faces; f_id++) {
+    cs_lnum_t s_id = mesh->i_face_vtx_idx[f_id];
+    cs_lnum_t e_id = mesh->i_face_vtx_idx[f_id+1];
+    for (cs_lnum_t i = s_id; i < e_id; i++) {
+      cs_lnum_t vtx_id = mesh->i_face_vtx_lst[i];
+      if (v_flag[vtx_id] != 0) {
+        cs_lnum_t c_id_0 = mesh->i_face_cells[f_id][0];
+        cs_lnum_t c_id_1 = mesh->i_face_cells[f_id][1];
+        cs_lnum_t j = _v2c_idx[vtx_id] + v2c_count[vtx_id];
+        if (c_id_0 > -1) {
+          _v2c[j++] = c_id_0;
+          v2c_count[vtx_id] += 1;
+        }
+        if (c_id_1 > -1) {
+          _v2c[j++] = c_id_1;
+          v2c_count[vtx_id] += 1;
+        }
+      }
+    }
+  }
+
+  for (cs_lnum_t f_id = 0; f_id < mesh->n_b_faces; f_id++) {
+    cs_lnum_t s_id = mesh->b_face_vtx_idx[f_id];
+    cs_lnum_t e_id = mesh->b_face_vtx_idx[f_id+1];
+    for (cs_lnum_t i = s_id; i < e_id; i++) {
+      cs_lnum_t vtx_id = mesh->b_face_vtx_lst[i];
+      if (v_flag[vtx_id] != 0) {
+        cs_lnum_t c_id_0 = mesh->b_face_cells[f_id];
+        cs_lnum_t j = _v2c_idx[vtx_id] + v2c_count[vtx_id];
+        _v2c[j] = c_id_0;
+        v2c_count[vtx_id] += 1;
+      }
+    }
+  }
+
+  BFT_FREE(v2c_count);
+
+  /* Order and compact adjacency array */
+
+  cs_sort_indexed(n_vertices, _v2c_idx, _v2c);
+
+  cs_lnum_t *tmp_v2c_idx = NULL;
+
+  BFT_MALLOC(tmp_v2c_idx, n_vertices+1, cs_lnum_t);
+  memcpy(tmp_v2c_idx, _v2c_idx, (n_vertices+1)*sizeof(cs_lnum_t));
+
+  cs_lnum_t k = 0;
+
+  for (cs_lnum_t i = 0; i < n_vertices; i++) {
+    cs_lnum_t js = tmp_v2c_idx[i];
+    cs_lnum_t je = tmp_v2c_idx[i+1];
+    cs_lnum_t v2c_prev = -1;
+    _v2c_idx[i] = k;
+    for (cs_lnum_t j = js; j < je; j++) {
+      if (v2c_prev != _v2c[j]) {
+        _v2c[k++] = _v2c[j];
+        v2c_prev = _v2c[j];
+      }
+    }
+  }
+  _v2c_idx[n_vertices] = k;
+
+  assert(_v2c_idx[n_vertices] <= tmp_v2c_idx[n_vertices]);
+
+  BFT_FREE(tmp_v2c_idx);
+  BFT_REALLOC(_v2c, _v2c_idx[n_vertices], cs_lnum_t);
+
+  *v2c_idx = _v2c_idx;
+  *v2c = _v2c;
+}
+
 /*----------------------------------------------------------------------------*/
 
 END_C_DECLS
